package com.hz.pm.api.common.handler;

import cn.hutool.core.date.StopWatch;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.system.oshi.OshiUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.lang.NonNull;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

/**
 * <p>
 * CpuLoadTask
 * </p>
 *
 * @author WendyYang
 * @since 20:30 2024/3/5
 */
@Slf4j
@Profile("prod")
@Configuration
public class CpuLoadTask {

    private static final int LIMIT_COUNT = 2000000000;
    private final AtomicInteger taskNo = new AtomicInteger(0);

    private static final Integer CPU_LIMIT = 20;

    private static final ExecutorService FIXED_THREAD_POOL;

    private static class CpuLoadThreadFactory implements ThreadFactory {

        private final int priority;

        public CpuLoadThreadFactory(int priority) {
            this.priority = priority;
        }

        @Override
        public Thread newThread(@NonNull Runnable r) {
            Thread thread = new Thread(r);
            thread.setPriority(priority);
            return thread;
        }

    }

    static {
        int logicalProcessorCount = OshiUtil.getProcessor().getLogicalProcessorCount();
        int max = Math.max(logicalProcessorCount * CPU_LIMIT / 100, 1);
        CpuLoadThreadFactory threadFactory = new CpuLoadThreadFactory(Thread.MIN_PRIORITY);
        FIXED_THREAD_POOL = Executors.newFixedThreadPool(max, threadFactory);
    }


    @Scheduled(fixedRate = 1, timeUnit = TimeUnit.MINUTES)
    public void run() {
        double cpuUsage = OshiUtil.getCpuInfo().getUsed();
        log.info("当前CPU使用率：{}", cpuUsage);
        if (cpuUsage < CPU_LIMIT) {
            log.info("提交计算任务：{}", taskNo.incrementAndGet());
            FIXED_THREAD_POOL.execute(() -> {
                StopWatch watch = new StopWatch();
                watch.start();
                long sum = Stream.generate(() -> RandomUtil.randomLong(1, LIMIT_COUNT))
                        .map(w -> w + LIMIT_COUNT / w)
                        .limit(LIMIT_COUNT)
                        .mapToLong(Long::longValue)
                        .sum();
                watch.stop();
                log.info("总计：" + sum + " 耗时：" + watch.getTotal(TimeUnit.SECONDS));
            });
        }
    }

}
