package com.hz.pm.api.common.handler;

import com.ningdatech.basic.enumeration.Status;
import com.ningdatech.basic.model.ApiResponse;
import com.ningdatech.basic.util.CollUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.validation.BindException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.NoHandlerFoundException;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.stream.Collectors;

/**
 * <p>
 * GlobalExceptionHandler
 * </p>
 *
 * @author WendyYang
 * @since 17:57 2024/6/6
 */
@Slf4j
@ControllerAdvice
@ResponseStatus(HttpStatus.BAD_REQUEST)
public class GlobalExceptionHandler {

    @ResponseBody
    @ExceptionHandler(value = NoHandlerFoundException.class)
    public ApiResponse<Void> noHandlerFoundException(NoHandlerFoundException e) {
        return ApiResponse.ofStatus(Status.REQUEST_NOT_FOUND);
    }

    @ResponseBody
    @ExceptionHandler(value = {MethodArgumentNotValidException.class, BindException.class})
    public ApiResponse<Void> bindException(BindException e) {
        String msg = e.getAllErrors().stream()
                .map(DefaultMessageSourceResolvable::getDefaultMessage)
                .collect(Collectors.joining(" "));
        return ApiResponse.of(Status.BAD_REQUEST.getCode(), msg);
    }

    @ResponseBody
    @ExceptionHandler(value = ConstraintViolationException.class)
    public ApiResponse<Void> constraintViolationException(ConstraintViolationException e) {
        String msg = CollUtils.join(e.getConstraintViolations(), ConstraintViolation::getMessage, " ");
        return ApiResponse.of(Status.BAD_REQUEST.getCode(), msg);
    }

    @ResponseBody
    @ExceptionHandler(value = AccessDeniedException.class)
    public ApiResponse<Void> accessDeniedException(AccessDeniedException e) {
        log.error("异常信息: {} ", e.getMessage(), e);
        return ApiResponse.of(Status.BAD_REQUEST.getCode(), "暂无操作权限");
    }

    @ResponseBody
    @ExceptionHandler(value = Exception.class)
    public ApiResponse<Void> handlerException(Exception e) {
        log.error("异常信息: {} ", e.getMessage(), e);
        return ApiResponse.of(Status.BAD_REQUEST.getCode(), "系统异常，请联系管理员");
    }

}
