package com.hz.pm.api.common.helper;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.StrUtil;
import com.hz.pm.api.external.MobileCallClient;
import com.hz.pm.api.external.MobileCallClient.ReplyCallDTO;
import com.hz.pm.api.external.MobileCallClient.SendCallDTO;
import com.hz.pm.api.external.model.dto.CallRetDTO;
import com.hz.pm.api.external.sms.MhSmsClient;
import com.hz.pm.api.external.sms.dto.SmsDTO;
import com.hz.pm.api.external.sms.vo.SmsReplyDTO;
import com.hz.pm.api.external.sms.vo.SmsSendDTO;
import com.hz.pm.api.sys.model.entity.MsgCallRecord;
import com.hz.pm.api.sys.model.enumeration.BizTypeEnum;
import com.hz.pm.api.sys.model.enumeration.SubmitTypeEnum;
import com.hz.pm.api.sys.service.IMsgCallRecordService;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static com.hz.pm.api.sys.model.entity.MsgCallRecord.ReplyStatus;
import static com.hz.pm.api.sys.model.entity.MsgCallRecord.SendStatus;

/**
 * <p>
 * MsgCallHelper
 * </p>
 *
 * @author WendyYang
 * @since 15:49 2024/4/25
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class MsgCallHelper {

    private final IMsgCallRecordService msgCallRecordService;

    /**
     * 发送短信
     *
     * @param phone   手机号
     * @param content 短信内容
     * @param bizType 业务类型
     * @return submitKey
     **/
    public String sendMsg(String phone, String content, BizTypeEnum bizType) {
        return sendMsg(Collections.singletonList(phone), content, bizType);
    }

    /**
     * 批量发送短信
     *
     * @param phones  手机号
     * @param content 短信内容
     * @param bizType 业务类型
     * @return submitKey
     **/
    public String sendMsg(Collection<String> phones, String content, BizTypeEnum bizType) {
        log.info("sendMsgCall:{},{}", phones, content);
        Assert.notEmpty(phones, "手机号不能为空");
        phones.forEach(w -> {
            if (!PhoneUtil.isMobile(w)) {
                throw BizException.wrap("手机号格式错误：%s", w);
            }
        });
        SmsDTO<SmsSendDTO> ret = MhSmsClient.getClient().smsSend(content, phones);
        if (!ret.isOk()) {
            log.error("短信发送失败:{}", ret);
            throw BizException.wrap("短信发送失败");
        }
        SmsSendDTO data = ret.getData();
        List<String> errorPhones = StrUtil.split(data.getErrorPhone(), ",");
        List<MsgCallRecord> records = phones.stream().map(w -> {
            MsgCallRecord mcr = new MsgCallRecord();
            mcr.setContent(content);
            mcr.setReceivePhone(w);
            mcr.setSubmitKey(data.getResult());
            mcr.setBizType(bizType);
            mcr.setSubmitType(SubmitTypeEnum.SMS);
            mcr.setSendStatus(errorPhones.contains(w) ? SendStatus.SUCCESS : SendStatus.FAILED);
            buildReplyStatusBySend(mcr, bizType);
            return mcr;
        }).collect(Collectors.toList());
        msgCallRecordService.saveBatch(records);
        return data.getResult();
    }

    private void buildReplyStatusBySend(MsgCallRecord mcr, BizTypeEnum bizType) {
        if (SendStatus.SUCCESS.equals(mcr.getSendStatus())) {
            switch (bizType) {
                case EXPERT_INVITE:
                    mcr.setNeedReply(Boolean.TRUE);
                    mcr.setReplyStatus(ReplyStatus.WAITING);
                    break;
                default:
                    mcr.setNeedReply(Boolean.FALSE);
                    break;
            }
        }
    }

    /**
     * 获取短信回复内容
     *
     * @param submitKey submitKey
     * @return 回复内容
     */
    public SmsReplyDTO smsReply(String submitKey) {
        return MhSmsClient.getClient().smsReply(submitKey);
    }

    /**
     * 批量提交电话任务
     *
     * @param phones  手机号
     * @param content 内容
     * @param bizType 业务类型
     * @return submitKey submitKey
     */
    public String sendCall(Collection<String> phones, String content, BizTypeEnum bizType) {
        log.info("sendCall:{},{}", phones, content);
        Assert.notEmpty(phones, "手机号不能为空");
        phones.forEach(phone -> {
            if (!PhoneUtil.isMobile(phone)) {
                throw BizException.wrap("手机号格式错误：%s", phone);
            }
        });
        CallRetDTO<SendCallDTO> ret = MobileCallClient.getClient().sendCall(phones, content);
        log.info("电话拨打：{} {}", phones, ret);
        if (!ret.isOk()) {
            throw BizException.wrap("电话拨打失败");
        }
        SendCallDTO data = ret.getData();
        List<MsgCallRecord> records = phones.stream().map(w -> {
            MsgCallRecord mcr = new MsgCallRecord();
            mcr.setContent(content);
            mcr.setReceivePhone(w);
            mcr.setSubmitKey(data.getRequestId());
            mcr.setBizType(bizType);
            mcr.setSubmitType(SubmitTypeEnum.CALL);
            mcr.setSendStatus(SendStatus.SUCCESS);
            buildReplyStatusBySend(mcr, bizType);
            return mcr;
        }).collect(Collectors.toList());
        msgCallRecordService.saveBatch(records);
        return data.getRequestId();
    }

    /**
     * 提交电话任务
     *
     * @param phone   手机号
     * @param content 内容
     * @param bizType 业务类型
     * @return submitKey submitKey
     */
    public String sendCall(String phone, String content, BizTypeEnum bizType) {
        return sendCall(Collections.singletonList(phone), content, bizType);
    }

    public List<ReplyCallDTO> callReply(String submitKey) {
        CallRetDTO<List<ReplyCallDTO>> retCallReplies = MobileCallClient.getClient().replyCall(submitKey);
        log.error("获取电话回执：{} {}", submitKey, retCallReplies);
        if (retCallReplies.isOk()) {
            List<ReplyCallDTO> replies = retCallReplies.getData();
            if (replies == null || replies.isEmpty()) {
                return Collections.emptyList();
            }
            return replies;
        }
        throw BizException.wrap("获取电话回执失败");
    }

}
