package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.AdaptStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderAdaptStatus;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * <p>
 * 标段适配改造状态机
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/2/24
 */
@Slf4j
@Component
@EnableStateMachine(name = AdaptStateMachineBuilderImpl.MACHINE_ID)
@RequiredArgsConstructor
public class AdaptStateMachineBuilderImpl implements BaseStateMachineBuilder<Purchase, AdaptStateChangeEvent, TenderAdaptStatus> {

    public static final String MACHINE_ID = "adaptStateMachine";

    private final BeanFactory beanFactory;

    @Override
    public StateMachine<TenderAdaptStatus, AdaptStateChangeEvent> build() throws StateMachineException {
        StateMachine<TenderAdaptStatus, AdaptStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：" + stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    @Override
    public StateMachine<TenderAdaptStatus, AdaptStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<TenderAdaptStatus, AdaptStateChangeEvent> buildStateMachine(BeanFactory factory) throws Exception {
        StateMachineBuilder.Builder<TenderAdaptStatus, AdaptStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(factory);

        builder.configureStates()
                .withStates()
                .initial(TenderAdaptStatus.WITHOUT_ADAPT_INFO)
                .states(EnumSet.allOf(TenderAdaptStatus.class));

        builder.configureTransitions()
                .withExternal()
                .source(TenderAdaptStatus.WITHOUT_ADAPT_INFO)
                .target(TenderAdaptStatus.ADAPT_INFO_AUDIT)
                .event(AdaptStateChangeEvent.SUBMIT_ADAPT_INFO)
                .and()
                .withExternal()
                .source(TenderAdaptStatus.ADAPT_INFO_AUDIT)
                .target(TenderAdaptStatus.ADAPT_INFO_PASSED)
                .event(AdaptStateChangeEvent.ADAPT_INFO_PASSED)
                .and()
                .withExternal()
                .source(TenderAdaptStatus.ADAPT_INFO_AUDIT)
                .target(TenderAdaptStatus.ADAPT_INFO_FAILED)
                .event(AdaptStateChangeEvent.ADAPT_INFO_FAILED)
                .and()
                .withExternal()
                .source(TenderAdaptStatus.ADAPT_INFO_FAILED)
                .target(TenderAdaptStatus.ADAPT_INFO_AUDIT)
                .event(AdaptStateChangeEvent.RESUBMIT_ADAPT_INFO)
                .and();
        return builder.build();
    }

    @Override
    @Bean(name = "adaptStatePersister")
    public StateMachinePersister<TenderAdaptStatus, AdaptStateChangeEvent, Purchase> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<TenderAdaptStatus, AdaptStateChangeEvent, Purchase>() {
            @Override
            public void write(StateMachineContext<TenderAdaptStatus, AdaptStateChangeEvent> context, Purchase contextObj) {
                log.info("当前项目为：{} ==> {}", contextObj.getProjectId(), contextObj.getId());
            }

            @Override
            public StateMachineContext<TenderAdaptStatus, AdaptStateChangeEvent> read(Purchase contextObj) {
                TenderAdaptStatus status = TenderAdaptStatus.getNoNull(contextObj.getAdaptStatus());
                return new DefaultStateMachineContext<>(status, null, null, null, null, MACHINE_ID);
            }
        });
    }

}
