package com.hz.pm.api.common.statemachine.util;

import com.hz.pm.api.common.model.constant.StateMachineConst;
import com.hz.pm.api.common.statemachine.builder.impl.ProjectStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.service.IProjectApplicationService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.function.Function;

/**
 * 状态机工具类
 *
 * @author CMM
 * @since 2023/02/07 22:49
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectStateMachineUtil implements AbstractStateMachineUtil<Project, ProjectStateChangeEvent> {

    private static final String PROJECT_DECLARE = StateMachineConst.PROJECT_DECLARE;

    private static final String APPLICATION_DECLARE = StateMachineConst.APPLICATION_DECLARE;

    //==================================================================================================================

    private final ProjectStateMachineBuilderImpl projectStateMachineBuilder;

    private final IProjectStatusChangeService projectStatusChangeService;

    private final IProjectApplicationService projectApplicationService;

    @Override
    public Function<Project, Integer> statusFunction() {
        return Project::getStatus;
    }

    @Override
    public void execute(Project project, ProjectStateChangeEvent event) throws Exception {
        log.info("调用状态机前的项目状态为：{}", project.getStatus());
        // 将状态变更记录保存到项目状态变更表中
        ProjectStatusChange projectStatusChange = new ProjectStatusChange();
        projectStatusChange.setBeforeStatus(project.getStatus());

        // 获取TO状态机
        StateMachine<ProjectStatus, ProjectStateChangeEvent> stateMachine = projectStateMachineBuilder.build();
        Message<ProjectStateChangeEvent> message = MessageBuilder.withPayload(event)
                .setHeader(PROJECT_DECLARE, project)
                .setHeader(APPLICATION_DECLARE, projectApplicationService.getApplicationsByProject(project))
                .build();
        //初始化状态机
        StateMachinePersister<ProjectStatus, ProjectStateChangeEvent, Project> projectDeclareStateMachinePersister = projectStateMachineBuilder.stateMachinePersister();
        projectDeclareStateMachinePersister.restore(stateMachine, project);
        stateMachine.sendEvent(message);
        projectStatusChange.setProjectId(project.getId());
        projectStatusChange.setAfterStatus(project.getStatus());
        projectStatusChange.setEvent(event.name());
        projectStatusChange.setCreateOn(LocalDateTime.now());
        projectStatusChange.setProjectCode(project.getProjectCode());
        // 插入项目状态变更表中
        projectStatusChangeService.save(projectStatusChange);
        log.info("调用状态机后的项目状态为：{}", project.getStatus());
    }

}
