package com.hz.pm.api.meeting.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.helper.MsgCallHelper;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.common.util.MDCThreadPoolTaskExecutor;
import com.hz.pm.api.external.model.enumeration.CallDialStatus;
import com.hz.pm.api.external.sms.vo.SmsReply;
import com.hz.pm.api.external.sms.vo.SmsReplyDTO;
import com.hz.pm.api.sys.model.entity.MsgCallRecord;
import com.hz.pm.api.sys.model.entity.MsgCallRecord.ReplyStatus;
import com.hz.pm.api.sys.model.entity.MsgCallRecord.SendStatus;
import com.hz.pm.api.sys.model.enumeration.SubmitTypeEnum;
import com.hz.pm.api.sys.service.IMsgCallRecordService;
import com.ningdatech.basic.util.CollUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import static com.hz.pm.api.external.MobileCallClient.ReplyCallDTO;

/**
 * <p>
 * MsgCallReplyRewriteTask
 * </p>
 *
 * @author WendyYang
 * @since 21:29 2024/4/25
 */
@Slf4j
@Component
@ConditionalOnProperty(prefix = "msg-call-reply-write", value = "open", havingValue = "true", matchIfMissing = true)
public class MsgCallReplyRewriteTask {

    private static final ThreadPoolTaskExecutor EXECUTOR;

    static {
        // 初始化线程池
        EXECUTOR = new MDCThreadPoolTaskExecutor();
        EXECUTOR.setCorePoolSize(2);
        EXECUTOR.setMaxPoolSize(4);
        EXECUTOR.setQueueCapacity(100);
        EXECUTOR.setKeepAliveSeconds(120);
        EXECUTOR.setThreadPriority(Thread.NORM_PRIORITY);
        EXECUTOR.setWaitForTasksToCompleteOnShutdown(true);
        EXECUTOR.setThreadNamePrefix("callRewriteExecutor-");
        EXECUTOR.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
    }

    private final IMsgCallRecordService msgCallRecordService;
    private final MsgCallHelper msgCallHelper;

    public MsgCallReplyRewriteTask(IMsgCallRecordService msgCallRecordService, MsgCallHelper msgCallHelper) {
        this.msgCallRecordService = msgCallRecordService;
        this.msgCallHelper = msgCallHelper;
        EXECUTOR.initialize();
    }


    @Scheduled(fixedRateString = "${msg-call-reply-write.fixed-rate:1}", timeUnit = TimeUnit.MINUTES)
    public void replyRewrite() {
        log.info("开始获取短信电话回执");
        Wrapper<MsgCallRecord> query = Wrappers.lambdaQuery(MsgCallRecord.class)
                .eq(MsgCallRecord::getNeedReply, Boolean.TRUE)
                .eq(MsgCallRecord::getSendStatus, SendStatus.SUCCESS)
                .eq(MsgCallRecord::getReplyStatus, ReplyStatus.WAITING)
                .isNull(MsgCallRecord::getReplyOn);
        List<MsgCallRecord> records = msgCallRecordService.list(query);
        if (records.isEmpty()) {
            return;
        }
        LocalDateTime startTime = LocalDateTime.now().minusMinutes(60);
        Map<Boolean, List<MsgCallRecord>> map = CollUtils.group(records, w -> w.getCreateOn().isBefore(startTime));
        List<MsgCallRecord> overtimeRecords = map.get(Boolean.TRUE);
        if (CollUtil.isNotEmpty(overtimeRecords)) {
            overtimeRecords.forEach(w -> {
                w.setReplyStatus(ReplyStatus.OVERTIME);
                w.setReplyOn(LocalDateTime.now());
            });
            msgCallRecordService.updateBatchById(overtimeRecords);
        }
        List<MsgCallRecord> waitingRecords = map.get(Boolean.FALSE);
        if (CollUtil.isNotEmpty(waitingRecords)) {
            CollUtils.group(waitingRecords, MsgCallRecord::getSubmitType)
                    .forEach((submitType, recordsBySummitType) -> {
                        if (SubmitTypeEnum.CALL.equals(submitType)) {
                            rewriteCallReply(recordsBySummitType);
                        } else if (SubmitTypeEnum.SMS.equals(submitType)) {
                            rewriteSmsReply(recordsBySummitType);
                        }
                    });
        }
    }

    /**
     * 获取短信回复结果
     *
     * @param records 短信记录
     */
    private void rewriteSmsReply(List<MsgCallRecord> records) {
        CollUtils.group(records, MsgCallRecord::getSubmitKey)
                .forEach((submitKey, currRecords) -> {
                    Runnable function = () -> {
                        try {
                            SmsReplyDTO retReply = msgCallHelper.smsReply(submitKey);
                            String errorPhoneStr = retReply.getErrorPhone();
                            List<String> errorPhones = StrUtil.split(errorPhoneStr, ",");
                            List<SmsReply> replies = retReply.getResult();
                            Map<String, SmsReply> currReplyMap = new HashMap<>();
                            if (CollUtil.isNotEmpty(replies)) {
                                replies.sort(Comparator.comparing(SmsReply::getReplyReplytime));
                                replies.forEach(w -> currReplyMap.putIfAbsent(w.getReplyMobile(), w));
                            }
                            List<MsgCallRecord> updates = new ArrayList<>();
                            for (MsgCallRecord w : currRecords) {
                                if (errorPhones.contains(w.getReceivePhone())) {
                                    w.setReplyStatus(ReplyStatus.FAILED);
                                    w.setReplyOn(LocalDateTime.now());
                                } else if (currReplyMap.containsKey(w.getReceivePhone())) {
                                    SmsReply reply = currReplyMap.get(w.getReceivePhone());
                                    w.setReplyOn(LocalDateTimeUtil.of(reply.getReplyReplytime()));
                                    w.setReplyContent(reply.getReplyContent());
                                    w.setReplyStatus(ReplyStatus.SUCCESS);
                                } else {
                                    continue;
                                }
                                updates.add(w);
                            }
                            msgCallRecordService.updateBatchById(updates);
                        } catch (Exception e) {
                            log.error("短信回复结果回填失败：{}", submitKey, e);
                        }
                    };
                    EXECUTOR.execute(function);
                });
    }

    /**
     * 获取电话回复结果
     *
     * @param records 电话记录
     **/
    private void rewriteCallReply(List<MsgCallRecord> records) {
        CollUtils.group(records, MsgCallRecord::getSubmitKey)
                .forEach((submitKey, currRecords) -> {
                    Runnable function = () -> {
                        try {
                            List<ReplyCallDTO> retReplies = msgCallHelper.callReply(submitKey);
                            Map<String, ReplyCallDTO> replyMap = BizUtils.groupFirstMap(retReplies,
                                    ReplyCallDTO::getRemoteNumber,
                                    Comparator.comparing(ReplyCallDTO::getSendDate));
                            List<MsgCallRecord> updates = new ArrayList<>();
                            for (MsgCallRecord w : currRecords) {
                                ReplyCallDTO reply = replyMap.get(w.getReceivePhone());
                                if (reply == null) {
                                    continue;
                                }
                                switch (CallDialStatus.getNoNull(reply.getDialStatus())) {
                                    case CALL_ENDED_CONNECTED:
                                        w.setReplyStatus(ReplyStatus.SUCCESS);
                                        w.setReplyContent(reply.getDtmf());
                                        w.setReplyOn(LocalDateTimeUtil.of(reply.getAnswerDate()));
                                        w.setDuration(reply.getDuration());
                                        break;
                                    case CALL_ENDED_DISCONNECTED:
                                        w.setReplyStatus(ReplyStatus.FAILED);
                                        w.setReplyOn(LocalDateTimeUtil.of(reply.getDialDate()));
                                        w.setReplyContent(reply.getDtmf());
                                        w.setErrorRemark(reply.getRemark());
                                        break;
                                    default:
                                        break;
                                }
                                if (!ReplyStatus.WAITING.equals(w.getReplyStatus())) {
                                    updates.add(w);
                                }
                            }
                            msgCallRecordService.updateBatchById(updates);
                        } catch (Exception e) {
                            log.error("电话回复结果回填失败：{}", submitKey, e);
                        }
                    };
                    EXECUTOR.execute(function);
                });
    }

}
