package com.hz.pm.api.sms.manage;

import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.RandomUtil;
import com.hz.pm.api.common.helper.MsgCallHelper;
import com.hz.pm.api.external.sms.MhSmsClient;
import com.hz.pm.api.sms.constant.VerificationCodeType;
import com.hz.pm.api.sms.constant.VoiceSmsTemplateConst;
import com.hz.pm.api.sms.model.dto.VerifyCodeCacheDTO;
import com.hz.pm.api.sms.model.po.ReqVerificationCodePO;
import com.hz.pm.api.sms.utils.DateUtil;
import com.hz.pm.api.sms.utils.SmsRedisKeyUtils;
import com.hz.pm.api.sys.model.enumeration.BizTypeEnum;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.cache.model.cache.CacheKey;
import com.ningdatech.cache.repository.CachePlusOps;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.Objects;

/**
 * @author liuxinxin
 * @date 2023/2/16 下午4:42
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class VerificationCodeManage {

    private final MsgCallHelper msgCallHelper;
    private final CachePlusOps cachePlusOps;

    public void sendVerificationCode(ReqVerificationCodePO req) {
        Assert.isTrue(PhoneUtil.isMobile(req.getMobile()), "手机号码格式不正确");
        String verificationType = req.getVerificationType();
        VerificationCodeType codeType = VerificationCodeType.of(verificationType);

        // 验证是否被锁定
        String lockKey = SmsRedisKeyUtils.smsSendLockKey(codeType, req.getMobile());
        if (StringUtils.isNotBlank(cachePlusOps.get(lockKey))) {
            throw BizException.wrap("今日" + codeType.getDesc() + "的验证码发送次数过多，已被锁定");
        }
        // 验证发送间隔
        String cacheKey = SmsRedisKeyUtils.smsCodeVerifyKey(codeType, req.getMobile());
        VerifyCodeCacheDTO preCache = cachePlusOps.get(cacheKey);
        if (preCache != null && (LocalDateTime.now().minusMinutes(codeType.getSendInterval())
                .isBefore(preCache.getSendTime()))) {
            throw BizException.wrap(codeType.getSendInterval() + "分钟之内已发送过验证码，请稍后重试");
        }
        String code = RandomUtil.randomNumbers(6);
        VerifyCodeCacheDTO cache = VerifyCodeCacheDTO.builder()
                .code(code)
                .sendTime(LocalDateTime.now())
                .mobile(req.getMobile())
                .build();

        // 创建短信内容
        switch (codeType) {
            case LOGIN: {
                String content = String.format(VoiceSmsTemplateConst.SMS_VERIFY_CODE, code, codeType.getExpireTime());
                msgCallHelper.sendMsg(content, req.getMobile(), BizTypeEnum.VERIFY_CODE);
            }
            break;
            case EXPERT_REGISTER: {
                String content = String.format(VoiceSmsTemplateConst.EXPERT_REGISTER, code, codeType.getExpireTime());
                msgCallHelper.sendMsg(content, req.getMobile(), BizTypeEnum.VERIFY_CODE);
            }
            break;
            default:
                throw new IllegalArgumentException("非法的短信发送类型");
        }

        // 发送 短信
        log.info("发送短信验证码：{} -> {}", req.getMobile(), code);

        cachePlusOps.set(new CacheKey(cacheKey, Duration.ofMinutes(codeType.getExpireTime())), cache);
        String limitKey = SmsRedisKeyUtils.smsSendLimitKey(codeType, req.getMobile());
        if (Objects.nonNull(cachePlusOps.get(limitKey))) {
            Integer limitCount = cachePlusOps.get(limitKey);
            cachePlusOps.set(new CacheKey(limitKey, Duration.ofSeconds(DateUtil.restSecondsFromNowToNoon())), limitCount++);
            // 超出单日发送次数之后直接锁定
            if (limitCount >= codeType.getSendTimesByDay().longValue()) {
                cachePlusOps.set(new CacheKey(lockKey, Duration.ofSeconds(DateUtil.restSecondsFromNowToNoon())), req.getMobile());
            }
        } else {
            cachePlusOps.set(new CacheKey(limitKey, Duration.ofSeconds(DateUtil.restSecondsFromNowToNoon())), 1);
        }
    }


}
