package com.hz.pm.api.user.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.external.model.enumeration.MhUnitTypeEnum;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.MhUnitTreeDTO;
import com.hz.pm.api.user.model.dto.MhUnitUserTreeDTO;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.model.entity.MhUnit;
import com.hz.pm.api.user.model.entity.UserInfo;
import com.hz.pm.api.user.model.enumeration.UnitUserNodeType;
import com.hz.pm.api.user.model.po.MhUnitListReq;
import com.hz.pm.api.user.model.po.UserTreeReq;
import com.hz.pm.api.user.model.vo.MhUnitListVO;
import com.hz.pm.api.user.service.IMhUnitService;
import com.hz.pm.api.user.service.IUserInfoService;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.TreeUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * MhUnitManage
 * </p>
 *
 * @author WendyYang
 * @since 15:33 2024/1/10
 */
@Component
@RequiredArgsConstructor
public class MhUnitManage {

    @Value("${mh.unit.root-parent-id:0}")
    private Long unitRootParentId;

    private final IMhUnitService mhUnitService;
    private final MhUnitCache mhUnitCache;
    private final IUserInfoService userInfoService;

    public List<MhUnitTreeDTO> getMhUnitTree(String name) {
        List<UnitDTO> retUnits;
        List<UnitDTO> allUnits = mhUnitCache.all();
        if (StrUtils.isNotBlank(name)) {
            allUnits.removeIf(w -> !w.getName().contains(name));
            retUnits = allUnits.stream()
                    .map(w -> mhUnitCache.getUnitIdPaths(w.getId()))
                    .flatMap(Collection::stream)
                    .distinct()
                    .map(mhUnitCache::getById)
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList());
        } else {
            retUnits = allUnits;
        }
        if (allUnits.isEmpty()) {
            return Collections.emptyList();
        }
        List<MhUnitTreeDTO> nodes = retUnits.stream().map(w -> {
            MhUnitTreeDTO node = BeanUtil.copyProperties(w, MhUnitTreeDTO.class);
            node.setStrip(node.getType().getStrip());
            return node;
        }).collect(Collectors.toList());
        return TreeUtil.convertToTree(nodes, unitRootParentId);
    }

    public List<MhUnitTreeDTO> getMhUnitStripTree() {
        List<UnitDTO> allUnits = mhUnitCache.all();
        List<MhUnitTreeDTO> nodes = allUnits.stream()
                .filter(w -> MhUnitTypeEnum.SASAC.equals(w.getType()) ||
                        ((MhUnitTypeEnum.REGION.equals(w.getType()) || MhUnitTypeEnum.NODE.equals(w.getType()))
                                && w.getName().endsWith("信创办")))
                .map(w -> BeanUtil.copyProperties(w, MhUnitTreeDTO.class))
                .collect(Collectors.toList());
        Function<MhUnitTreeDTO, Integer> function = w -> w.getType().getStrip().getCode();
        nodes.sort(Comparator.comparing(MhUnitTreeDTO::getName)
                .thenComparing(function, Comparator.reverseOrder()));
        return nodes;
    }

    public List<MhUnitTreeDTO> getMhUnitDashboardOption() {
        List<UnitDTO> allUnits = mhUnitCache.all();
        List<MhUnitTreeDTO> nodes = allUnits.stream()
                .filter(w -> MhUnitTypeEnum.SASAC.equals(w.getType()) ||
                        ((MhUnitTypeEnum.REGION.equals(w.getType()) || MhUnitTypeEnum.NODE.equals(w.getType()))
                                && (w.getName().endsWith("信创办") || "市级部门".equals(w.getName()) || "杭州市".equals(w.getName()))))
                .map(w -> BeanUtil.copyProperties(w, MhUnitTreeDTO.class))
                .collect(Collectors.toList());
        Function<MhUnitTreeDTO, Integer> function = w -> w.getType().getStrip().getCode();
        nodes.sort(Comparator.comparing(MhUnitTreeDTO::getName)
                .thenComparing(function, Comparator.reverseOrder()));
        return nodes;
    }

    public PageVo<MhUnitListVO> page(MhUnitListReq req) {
        LambdaQueryWrapper<MhUnit> query = Wrappers.lambdaQuery(MhUnit.class);
        if (StrUtils.isNotBlank(req.getName())) {
            query.and(q1 -> q1.like(MhUnit::getName, req.getName())
                    .or(q2 -> q2.like(MhUnit::getShortName, req.getName())));
        }

        Page<MhUnit> page = mhUnitService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<MhUnitListVO> records = page.getRecords().stream().map(w -> {
            MhUnitListVO unit = BeanUtil.copyProperties(w, MhUnitListVO.class, "type");
            MhUnitTypeEnum.getByCode(w.getType()).ifPresent(type -> {
                unit.setType(type);
                unit.setStrip(type.getStrip());
                unit.setTypeName(type.getVal());
                unit.setStripName(type.getStrip().getVal());
            });
            return unit;
        }).collect(Collectors.toList());
        return PageVo.of(records, page.getTotal());
    }

    public List<MhUnitUserTreeDTO> getUnitUserTree(UserTreeReq req) {
        List<MhUnitUserTreeDTO> selectedUnits = new ArrayList<>();
        Set<Long> viewUnitIds = new HashSet<>();
        boolean hasId = req.getId() != null;
        if (hasId) {
            List<Long> childIds = mhUnitCache.getViewChildIdsRecursion(req.getId());
            viewUnitIds.addAll(childIds);
        }
        boolean hasName = CharSequenceUtil.isNotBlank(req.getName());
        LambdaQueryWrapper<UserInfo> userQuery = Wrappers.lambdaQuery(UserInfo.class)
                .select(UserInfo::getMhUnitId, UserInfo::getId, UserInfo::getRealName)
                .in(!viewUnitIds.isEmpty(), UserInfo::getMhUnitId, viewUnitIds)
                .gt(UserInfo::getMhUnitId, 0);
        if (hasName) {
            // 查询满足名字的
            userQuery.like(UserInfo::getRealName, req.getName())
                    .last("limit 500");
        } else if (hasId) {
            List<UnitDTO> units = mhUnitCache.listByIds(viewUnitIds);
            selectedUnits.addAll(CollUtils.convert(units, this::getMhUnitUserTree));
        } else {
            List<UnitDTO> all = mhUnitCache.all();
            selectedUnits.addAll(CollUtils.convert(all, this::getMhUnitUserTree));
        }
        List<UserInfo> userInfos = userInfoService.list(userQuery);
        if (userInfos.isEmpty()) {
            return Collections.emptyList();
        }
        if (hasName) {
            Set<Long> tmpUnitIds = CollUtils.fieldSet(userInfos, UserInfo::getMhUnitId);
            Set<Long> allUnitIds = new HashSet<>();
            for (Long unitId : tmpUnitIds) {
                allUnitIds.addAll(mhUnitCache.getUnitIdPaths(unitId));
            }
            List<UnitDTO> allUnits = mhUnitCache.listByIds(allUnitIds);
            selectedUnits.addAll(CollUtils.convert(allUnits, this::getMhUnitUserTree));
        }
        selectedUnits.addAll(CollUtils.convert(userInfos, this::getMhUnitUserTree));
        String rootId = UnitUserNodeType.UNIT.name() + "#" + (hasId ? req.getId() : unitRootParentId);
        return TreeUtil.convertToTree(selectedUnits, rootId);
    }

    private MhUnitUserTreeDTO getMhUnitUserTree(UnitDTO unit) {
        MhUnitUserTreeDTO node = new MhUnitUserTreeDTO();
        node.setSort(unit.getSort());
        node.setName(unit.getName());
        node.setName(unit.getShortName());
        node.setBizId(unit.getId());
        node.setParentId(UnitUserNodeType.UNIT.name() + "#" + unit.getParentId());
        node.setId(UnitUserNodeType.UNIT.name() + "#" + unit.getId());
        node.setType(UnitUserNodeType.UNIT);
        return node;
    }

    private MhUnitUserTreeDTO getMhUnitUserTree(UserInfo w) {
        MhUnitUserTreeDTO node = new MhUnitUserTreeDTO();
        node.setSort((short) 1);
        node.setName(w.getRealName());
        node.setBizId(w.getId());
        node.setParentId(UnitUserNodeType.UNIT.name() + "#" + w.getMhUnitId());
        node.setId(UnitUserNodeType.USER.name() + "#" + w.getId());
        node.setType(UnitUserNodeType.USER);
        return node;
    }

}
